const DELAY_MS = 1000;
const playerStats = {
  Markus: {
    user: 'Markus',
    win: 3,
    lost: 6,
  },
  Michael: {
    user: 'Michael',
    win: 4,
    lost: 5,
  },
  Lisa: {
    user: 'Lisa',
    win: 4,
    lost: 5,
  },
};

function getRankingsFromPlayerStats(stats) {
  const ranking = [];
  const playerValues = Object.values(stats);
  playerValues.sort((a, b) => b.win - a.win);

  for (const playerValue of playerValues) {
    const newRank = ranking.length + 1;

    // eslint-disable-next-line @web-and-design/wed/use-action-map
    if (ranking.length === 0) {
      ranking.push({rank: newRank, wins: playerValue.win, players: [playerValue.user]});
    } else if (ranking[ranking.length - 1].wins === playerValue.win) {
      ranking[ranking.length - 1].players.push(playerValue.user);
    } else {
      ranking.push({rank: newRank, wins: playerValue.win, players: [playerValue.user]});
    }
  }

  return ranking;
}

export const HANDS = ['Rock', 'Paper', 'Scissors', 'Well', 'Matchstick'];

let isConnectedState = false;

export function setConnected(newIsConnected) {
  isConnectedState = Boolean(newIsConnected);
}

export function isConnected() {
  return isConnectedState;
}

export function getRankings(rankingsCallbackHandlerFn) {
  if (isConnected()) {
    fetch('https://stone.sifs0005.infs.ch/ranking')
      .then((res) => res.json())
      .then((json) => getRankingsFromPlayerStats(json))
      .then((rankings) => rankingsCallbackHandlerFn(rankings));
  } else {
    const rankings = getRankingsFromPlayerStats(playerStats);
    rankingsCallbackHandlerFn(rankings);
  }
}

const evalLookup = {
  scissors: {
    scissors: 0,
    rock: -1,
    paper: 1,
    well: -1,
    matchstick: 1,
  },
  rock: {
    scissors: 1,
    rock: 0,
    paper: -1,
    well: -1,
    matchstick: 1,
  },
  paper: {
    scissors: -1,
    rock: 1,
    paper: 0,
    well: 1,
    matchstick: -1,
  },
  well: {
    scissors: 1,
    rock: 1,
    paper: -1,
    well: 0,
    matchstick: -1,
  },
  matchstick: {
    scissors: -1,
    rock: -1,
    paper: 1,
    well: 1,
    matchstick: 0,
  },
};

const translationLookup = {
  Rock: 'Stein',
  Paper: 'Papier',
  Scissors: 'Schere',
  Well: 'Quelle',
  Matchstick: 'Streichholz',
};

function enDeTranslation(key) {
  return translationLookup[key];
}

function deEnTranslation(key) {
  const reversed = {};
  for (const k of Object.keys(translationLookup)) {
    reversed[translationLookup[k]] = k;
  }
  return reversed[key];
}

function getGameEval(playerHand, systemHand) {
  return evalLookup[playerHand.toLowerCase()][systemHand.toLowerCase()];
}

function reevaluateHandsFromServer(playerHand, systemHand, gameEval) {
  // if gameEval === false => player has lost OR draw
  if (!gameEval) {
    return getGameEval(playerHand, systemHand);
  }
  return 1; // if gameEval === true => player has won
}

function evaluateGame(playerHand, systemHand, gameEval, gameRecordHandlerCallbackFn) {
  setTimeout(() => gameRecordHandlerCallbackFn({
    playerHand,
    systemHand,
    gameEval,
  }), DELAY_MS);
}

function evaluateFromServer(playerHand, systemHand, gameEval, gameRecordHandlerCallbackFn) {
  const gameEvalAsNumber = reevaluateHandsFromServer(playerHand, systemHand, gameEval);
  evaluateGame(playerHand, systemHand, gameEvalAsNumber, gameRecordHandlerCallbackFn);
}

export function evaluateHand(playerName, playerHand, gameRecordHandlerCallbackFn) {
  // eslint-disable-next-line @web-and-design/wed/use-action-map
  if (isConnected()) {
    const translatedPlayerHand = enDeTranslation(playerHand);
    const url = `https://stone.sifs0005.infs.ch/play?playerName=${playerName}&playerHand=${translatedPlayerHand}`;
    fetch(url)
        .then((res) => res.json())
        .then((json) => evaluateFromServer(playerHand, deEnTranslation(json.choice),
            json.win, gameRecordHandlerCallbackFn));
  } else {
    const systemHand = HANDS[Math.floor(Math.random() * 5)];
    const gameEval = getGameEval(playerHand, systemHand);

    // eslint-disable-next-line @web-and-design/wed/use-action-map
    if (gameEval > 0) {
      playerStats[playerName].win += 1;
    } else if (gameEval < 0) {
      playerStats[playerName].lost += 1;
    }

    evaluateGame(playerHand, systemHand, gameEval, gameRecordHandlerCallbackFn);
  }
}

export function createUserIfNotExists(username) {
  if (playerStats[username] === undefined) {
    playerStats[username] = {user: username, win: 0, lost: 0};
  }
}
